#!/usr/bin/env python 
'''
Copyright (C) 2008 Thomas Holder, http://sf.net/users/speleo3/
Distributed under the terms of the GNU General Public License v2

This program was inspired by http://www.cavediving.de/svg2th2.py
'''

try:
	import inkex, simplepath, simpletransform
except:
	# speleoex adds inkscape extensions dir to PYTHONPATH
	import speleoex
	import inkex, simplepath, simpletransform

class Th2Line:
	def __init__(self):
		self.options = {}
		self.points = []
	def append(self, params):
		self.points.append(" ".join(str(i) for i in params))
	def close(self):
		self.options['close'] = 'on'
	def output(self, linetype = 'wall'):
		line_cmd = "line " + linetype
		for k,v in self.options.iteritems():
			line_cmd += " -%s %s" % (k,v)
		print line_cmd
		print "  " + "\n  ".join(self.points)
		print "endline\n"

class Th2Output(inkex.Effect):
	def __init__(self):
		inkex.Effect.__init__(self)
		self.OptionParser.add_option("--scale",    type="int",    dest="scale",    default=100)
		self.OptionParser.add_option("--dpi",      type="int",    dest="dpi",      default=90)
		self.OptionParser.add_option("--linetype", type="string", dest="linetype", default="wall")
		self.OptionParser.add_option("--layers",   type="string", dest="layers",   default="all")

	def getTransform(self, node):
		m2 = simpletransform.parseTransform(node.get('transform'))
		while True:
			node = node.getparent()
			if not node:
				break
			t1 = node.get('transform')
			if not t1:
				continue
			m1 = simpletransform.parseTransform(t1)
			m2 = simpletransform.composeTransform(m1, m2)
		m2 = simpletransform.composeTransform(self.flip, m2)
		return m2
	
	def transformParams(self, mat, params):
		new = []
		for i in range(0, len(params), 2):
			new.append(mat[0][0]*params[i]+mat[0][1]*params[i+1]+mat[0][2])
			new.append(mat[1][0]*params[i]+mat[1][1]*params[i+1]+mat[1][2])
		return new

	def output(self):
		doc_width = inkex.unittouu(self.document.getroot().get('width'))
		doc_height = inkex.unittouu(self.document.getroot().get('height'))

		self.flip = [[1, 0, 0],[0, -1, doc_height]]

		print '''encoding  utf-8
##XTHERION## xth_me_area_adjust 0 0 %f %f
##XTHERION## xth_me_area_zoom_to 50
''' % (doc_width, doc_height)

		images = self.document.xpath('/svg:svg/svg:g//svg:image', namespaces=inkex.NSS)
		for node in reversed(images):
			params = [ inkex.unittouu(node.get('x')), inkex.unittouu(node.get('y')) ]
			mat = self.getTransform(node)
			params = self.transformParams(mat, params)
			href = node.get(inkex.addNS('href', 'xlink'))
			print '##XTHERION## xth_me_image_insert {%f 1 1.0} {%f {}} "%s" 0 {}' % \
				(params[0], params[1], href)

		meters_per_inch = 0.0254
		meters_per_unit = self.options.scale * meters_per_inch / self.options.dpi
		print '\nscrap scrap1 -scale [ %f ]\n' % (meters_per_unit)

		if self.options.layers == 'current':
			paths = self.current_layer.xpath('.//svg:path', namespaces=inkex.NSS)
		else:
			paths = self.document.xpath('/svg:svg/svg:g//svg:path', namespaces=inkex.NSS)

		for node in paths:
			d = node.get('d')
			p = simplepath.parsePath(d)

			mat = self.getTransform(node)

			th2line = None
			for cmd,params in p:
				if cmd == 'M':
					if th2line != None:
						th2line.output(self.options.linetype)
					th2line = Th2Line()
				if cmd == 'Z':
					th2line.close()
				else:
					th2line.append(self.transformParams(mat, params))
			if th2line != None:
				th2line.output()

		print "endscrap\n"

e = Th2Output()
e.affect()
